<?php

/*
   Julian Day Number Table for any year
   on the Julian or Gregorian Calendar.

   This program is designed to be stand-alone
   and has all the necessary PHPSL functions
   built-in for transportability.

   Program by Jay Tanner

   Author  : Jay Tanner 2023
   License : Public Domain
*/

/* -------------------------------------
   Initialize output buffer this must be
   the very first command of the program
   when using cookies.
*/
   ob_start();

   $TextArea1Text = '';   $TextArea1Rows = 4;
   $ErrorString = '';

// Set calendar year range limits.
   $YMin = '-4714';
   $YMax = '+9999';

// Set text area text and background colors.
// These will change to white/red on error.
   $TxColor = 'black';
   $BgColor = 'white';

// Set time zone offset to New York City, USA.
   date_default_timezone_set('America/New_York');
   $STZStr = date_default_timezone_get();

// ----------------------------------
// Program version and revision date.
   $_THIS_SCRIPT_NAME_ = BaseName($_SERVER["SCRIPT_FILENAME"]);
   $at  = "&#97;&#116;";   $UTC = "&#85;&#84;&#67;";
   $_REVISION_DATE_ = 'Revised: '.gmdate("l - F d, Y  $at H:i $UTC", FileMTime($_THIS_SCRIPT_NAME_));


// -----------------------------------
// Get current year from system clock.
   $cYear = date("Y");

   $_RUN_ = $_SERVER["SCRIPT_NAME"];

// Define texts for internal HTML title and page header.
   $_INTERNAL_HTML_TITLE_ = "JD Number Table For Any Given Month";

   $_PAGE_HEADER_TEXT_ = "<b style='font-weight:normal; font-size:110%'>JD Number and Julian Date Table For Any Month<br>On the Old Julian or Modern Gregorian Calendar</b><br>
   <span style='font-size:12px;'>Year&nbsp;Range&nbsp;&nbsp;=&nbsp;&nbsp;BC 4714&nbsp;&nbsp;to&nbsp;&nbsp;AD&nbsp;9999&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&minus;Neg&nbsp;|&nbsp;+Pos&nbsp;&nbsp;=&nbsp;&nbsp;BC&nbsp;|&nbsp;AD<br><br>PHP Program by Jay Tanner</span>";

/* ----------------------------------------------------
   Define a name for the cookie and set expiration date
   to 30 days if not used or deleted before then.  This
   value is expressed in seconds from the time that the
   cookie is created (Time Now + days*86400 seconds).
*/
   $CookieName = 'JD_Number_Table_Settings';

   $SetToExpireIn30Days = time() + 30*86400;

   $_COOKIE_FOUND_ = isset($_COOKIE[$CookieName]);

   if (!isset($_POST['ComputeButton']))
{

/* If there is a previous cookie, then restore
   the interface settings from it.
*/
   if ($_COOKIE_FOUND_)
  {
   $CookieDataString = $_COOKIE[$CookieName];
   list($Y, $JG, $m) = preg_split("[\|]", $CookieDataString);

   $Y = intval(trim($Y));
  }

else

/* If there is no previous cookie, then set default
   interface values and store them in a new cookie.
*/
  {
   $Y = date('Y');
   $JG = 'G';
   $m = date('m');
   $JGStr = ($JG == 'J')? 'Julian' : 'Gregorian';

   $CookieDataString = "$Y|$JG|$m";

   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
}



   if (isset($_POST['ComputeButton']))
{
   $Y = @trim($_POST['Y']);
   if ($Y == '') {$Y = date('Y');}
   $JG = @trim($_POST['JG']);
   $JG = (strtoupper(substr(trim($JG), 0,1)) == 'J')? 'J':'G';
   $JGStr = ($JG == 'J')? 'Julian' : 'Gregorian';

   $m = @trim($_POST['m']);  if (trim($m) == '') {$m = date('m');}
   $m = IntVal($m);

   $YMin = ($JG == 'J')? -4713 : -4714;

// _________________________
// Tidy up interface values.

   $Y = intval($Y);


// __________________________________
// Save interface settings to cookie.

   $CookieDataString = "$Y|$JG|$m";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
}

// _____________________
// Check for year error.

   if ($Y < $YMin or $Y == 0 or $Y > $YMax)
     {
       $TxColor = 'white';
       $BgColor = '#DD0000';

       $YZeroStr = '';
       if ($Y == 0)
       {$YZeroStr = "However, there is no\ncalendar year 0 (zero), by definition.";}

   $ErrorString =
"ERROR:
Invalid year value ($Y)

The year value must be an integer in the
range from $YMin to $YMax when using the
$JGStr calendar. $YZeroStr

BC years are entered as negative values.
";
      $TextArea1Text .= "$ErrorString";
      $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
     }


// Check for invalid month value.
   if ($m < 1 or $m == 0 or $m > 12)
     {
       $TxColor = 'white';
       $BgColor = '#DD0000';

   $ErrorString =
"

ERROR:
Invalid month value ($m)

The month value must be an integer in the
range from 1 = January to 12 = December.
";
      $TextArea1Text .= "$ErrorString";
      $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
     }












/*
   _______________________________________________________
   If no errors detected, then perform computations using
   given arguments and display the results in TEXTAREA1.
*/

   if (trim($ErrorString) == '')
  {
   $JDTableText   = JD_Num_Month_Table($Y, $m, $JG);
   $TextArea1Text = $JDTableText;
   $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
  }





print <<< _HTML

<!DOCTYPE HTML>

<HTML>

<head>

<title>JD Number Table For Any Given Month</title>
<meta name='viewport' content='width=device-width, initial-scale=1'>
<meta http-equiv='content-type' content='text/html; charset=UTF-8'>
<meta name='description' content='JD (Julian Day) Number Table Calculator For Any Given Month.'>
<meta name='keywords' content='JD number, Julian Day Number, Julian Date,calendar'>
<meta name='author' content='Jay Tanner - PHP Science Labs'>
<meta http-equiv='pragma'  content='no-cache'>
<meta http-equiv='expires' content='-1'>
<meta name='robots'    content='index,follow'>
<meta name='googlebot' content='index,follow'>
<link rel="canonical" href="https://www.phpsciencelabs.com/jd-number-table-for-any-month/">

<style type='text/css'>
body {color:white; background-color:black;  font-family:Verdana; font-size:15px}

table {font-family:Verdana; font-size:15px; border: 1px solid black;}

td {padding:8px;}

TextArea {color:black; background-color:white; font-family:monospace; font-size:14px; font-weight:bold; border:1px solid black; border-radius:8px; padding:4px; white-space:pre;}

input[type="text"]::-ms-clear {width:0; height:0;}
input[type="text"] {color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; border:2px solid black; border-radius:8px; box-shadow:4px 4px 3px #999999;}
input[type="text"]:focus {background-color:white; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold; border-radius:8px; box-shadow:4px 4px 3px #999999;}

input[type="submit"]{font-weight:bold;}
input[type="submit"]:hover{box-shadow:2px 2px 5px #222222; font-weight:bold;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

</head>

<body style='color:white; background-color:black;'>

<form ID='SmoothScroll' name='form1' method='post' action="$_RUN_">
<table align='left' width="550" style='font-size:14;'>

<tr><td style='color:white; background-color:#000066; text-align:center; border:2px solid white; border-radius:8px 8px 0px 0px;'>$_PAGE_HEADER_TEXT_</td></tr>

<tr><td style='color:black; background-color:white; text-align:center;'>
<b>Year <input type='text' name='Y' size='6'  maxlength='5' value="$Y" title=' -Neg = BC  |  +Pos = AD '>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
Month</b>&nbsp;<input type='text' name='m' size='3'  maxlength='2' value="$m" title=' Month = 1 to 12 '>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
<b>Calendar Mode</b> <input type='text' name='JG' size='3'  maxlength='2' value="$JG" title=' J = Julian | G = Gregorian '>

</td></tr>



<tr><td style='color:silver; background-color:black; text-align:center;'><br>
<input type='submit' name='ComputeButton' value='C O M P U T E'>
</td></tr>

<tr><td style='color:silver; background-color:black; text-align:center;'><br>
Double-Click to Select ALL Text<br><br>
$TextArea1Text
</td></tr>

<tr><td style="font-size:12px; color:#888888; background-color:black; text-align:center;">Program by Jay Tanner - $_REVISION_DATE_
<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>
</td></tr>

</table>
</form>

<script src='Smooth_Scroll.js'></script>
<script>SmoothScroll.init({cancelOnTouch: true});</script>

</body>

</html>


_HTML;



// *******************************************************
// *******************************************************
// *******************************************************
// *******************************************************

   function Days_in_Month ($Year, $Month, $JGMode='G')
{
   $Y = intval(trim($Year));  if ($Y < -4714 or $Y > 9999) {return FALSE;}
   $m = intval(trim($Month)); if ($m < 1 or $m > 12) {return FALSE;}
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   $mDays = substr("312831303130313130313031", 2*($m-1), 2);
   if (Is_Leap_Year($Y, $JG) and $m == 2) {return $mDays + 1;}
   return $mDays;
}




    function Is_Leap_Year ($Year, $JGMode='G')
{
   $Y = trim($Year);   $Y += ($Y < 0)? 1:0;
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   if ($JG == 'J' and $Y % 4 == 0) {return TRUE;}
   return ($Y % 4 == 0 and !($Y % 100 == 0 and $Y % 400 <> 0));
}




   function JD_Num ($Year, $Month, $Day, $JGMode='G')
{
   $Y = intval(trim($Year));
   $m = intval(trim($Month));
   $d = intval(trim($Day));
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0 : 1;
   $a = floor((14 - $m) / 12);
   $b = (($Y < 0)? $Y+1 : $Y) - $a;
   $c = floor($b/100);
   $J = floor(30.6001*(12*$a + $m + 1))
      + floor(365.25*($b + 4716)) + $d - 1524
      + $M*(floor($c/4) - $c + 2);
   return ("$Y $m $d" <> Inv_JD_Num($J, $JGMode))? FALSE : $J;
}



   function Inv_JD_Num ($JDNumber, $JGMode='G', $RetMode123=1)
{
   $w = intval(trim($RetMode123)); $RMode = ($w < 1 or $w > 3)? 1 : $w;
   $w = trim($JDNumber); if(!is_numeric($w)) {return FALSE;}
   $J = floor($w + 0.5); if($J < 0) {return FALSE;}
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0:1;
   $e = $M*floor(($J - 1867216.25)/36524.25);
   $f = $J + $M*($e - floor($e/4) + 1);
   $g = $f + 1524;
   $h = floor(($g - 122.1)/365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i)/30.6001);
   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $w = $h + floor((14-$m)/12) - 4716;
   $Y = ($w > 0)? $w : $w-1;

   $w   = '';
   $Mmm = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoW = substr('SunMonTueWedThuFriSat', 3*(($J+1) % 7), 3);
   $Y   = substr('     '.sprintf("%+05d", $Y), -5);

   switch ($RMode)
  {
   case  1: $w = intval(trim($Y))." $m $d"; break;
   case  2: $w = "$Y ".sprintf("%02d", $m).' '.sprintf("%02d", $d); break;
   case  3: $w = "$Y $Mmm ".sprintf("%02d", $d)." $DoW"; break;
   default: $w = 'RetMode Switch Error'; break;
  }
   return $w;
}








   function JD_Num_Month_Table ($BCADYearNum, $MonthNum, $JGCalMode='G')
{
   $y = IntVal(trim($BCADYearNum));
   $m = IntVal(trim($MonthNum));
   $JGMode = (StrToUpper(substr(trim($JGCalMode),0,1)) <> 'J')? 'G':'J';

   $YYYY = substr(abs($y).'    ', 0,4);

// Handle Gregorian calendar special case of November -4714.
   if ($JGMode == 'G' and $y == -4714 and $m == 11)
  {

  $wTable =
"  JULIAN DAY NUMBER AND JULIAN DATE TABLE

  For the Month of November, Year BC 4714
  On the Gregorian Calendar

-------------------------------------------
     Gregorian      JD Number   Julian Date
   Calendar Date    JD at 12h    JD at 00h
------------------  ---------   -----------
BC 4714-Nov-24-Mon         0         -0.5
BC 4714-Nov-25-Tue         1          0.5
BC 4714-Nov-26-Wed         2          1.5
BC 4714-Nov-27-Thu         3          2.5
BC 4714-Nov-28-Fri         4          3.5
BC 4714-Nov-29-Sat         5          4.5
BC 4714-Nov-30-Sun         6          5.5
------------------  ---------   -----------
";

// Adjust number of rows to use for current TEXTAREA.
   $TextRows = 19;

$TextArea =
"<!DOCTYPE HTML>

<textarea rows='$TextRows' cols='44' ReadOnly
 title=' Double-Click to Select ALL Text '
 OnDblClick='this.select();' OnMouseUp='return true;'
 style='font-size:130%; font-weight:bold; padding:6px;
 border:2px solid blue; border-radius:8px;'>
$wTable</textarea>

";

   return $TextArea;

}

// Handle Julian calendar special case of January -4713.
  if ($JGMode == 'J' and $y == -4713 and $m == 1)
  {
  $wTable =
"  JULIAN DAY NUMBER AND JULIAN DATE TABLE

  For the Month of January, Year BC 4713
  On the Gregorian Calendar

-------------------------------------------
    Gregorian       JD Number   Julian Date
  Calendar Date     JD at 12h    JD at 00h
------------------  ---------   -----------
BC 4713-Jan-01-Mon         0         -0.5
BC 4713-Jan-02-Tue         1          0.5
BC 4713-Jan-03-Wed         2          1.5
BC 4713-Jan-04-Thu         3          2.5
BC 4713-Jan-05-Fri         4          3.5
BC 4713-Jan-06-Sat         5          4.5
BC 4713-Jan-07-Sun         6          5.5
BC 4713-Jan-08-Mon         7          6.5
BC 4713-Jan-09-Tue         8          7.5
BC 4713-Jan-10-Wed         9          8.5
BC 4713-Jan-11-Thu        10          9.5
BC 4713-Jan-12-Fri        11         10.5
BC 4713-Jan-13-Sat        12         11.5
BC 4713-Jan-14-Sun        13         12.5
BC 4713-Jan-15-Mon        14         13.5
BC 4713-Jan-16-Tue        15         14.5
BC 4713-Jan-17-Wed        16         15.5
BC 4713-Jan-18-Thu        17         16.5
BC 4713-Jan-19-Fri        18         17.5
BC 4713-Jan-20-Sat        19         18.5
BC 4713-Jan-21-Sun        20         19.5
BC 4713-Jan-22-Mon        21         20.5
BC 4713-Jan-23-Tue        22         21.5
BC 4713-Jan-24-Wed        23         22.5
BC 4713-Jan-25-Thu        24         23.5
BC 4713-Jan-26-Fri        25         24.5
BC 4713-Jan-27-Sat        26         25.5
BC 4713-Jan-28-Sun        27         26.5
BC 4713-Jan-29-Mon        28         27.5
BC 4713-Jan-30-Tue        29         28.5
BC 4713-Jan-31-Wed        30         29.5
------------------  ---------   -----------
   Gregorian        JD at 12h    JD at 00h
 Calendar Date      JD Number   Julian Date
";

$TextRows = 45;

$TextArea =
"<!DOCTYPE HTML>

<textarea rows='$TextRows' cols='44' ReadOnly
 title=' Double-Click to Select ALL Text '
 OnDblClick='this.select();' OnMouseUp='return true;'
 style='font-size:130%; font-weight:bold; padding:6px;
 border:2px solid blue; border-radius:8px;'>
$wTable</textarea>

";

   return $TextArea;
}


// Compute JDNum of last day of previous month according to JGMode.
   $JDNum0 = ($JGMode == 'G')? GregorianToJD($m, 1, $y)-1
                             : JulianToJD($m, 1, $y)-1;

// Get full name of the month string (January to December) according to JGMode.
   $mName = ($JGMode == 'G')? JDMonthName($JDNum0+1, CAL_MONTH_GREGORIAN_LONG)
                            : JDMonthName($JDNum0+1, CAL_MONTH_JULIAN_LONG);

// Get 3-letter month name abbreviation string.
   $Mmm = substr($mName,0,3);

// Determine number of days in month according to JGMode.
   $mDays = ($JGMode == 'G')? Cal_Days_In_Month(CAL_GREGORIAN, $m, $y)
                            : Cal_Days_In_Month(CAL_JULIAN, $m, $y);

// Generate JD Number and Julian Date table string for the entire month.
   $wTable = '';
   for ($day=1;   $day <= $mDays;   $day++)
{
// Compute JD Number and Julian Date.
   $JDNum = $JDNum0 + $day;
   $JD = $JDNum - 0.5;

   $dd = SPrintF("%02d", $day);

// Determine day of the week 3-letter abbreviation string according to JDNum.
   $DoWStr = JDDayOfWeek($JDNum, 2);

// Format spacing of JD Number and JD.  JJNum 0000000 0000000.0
   $JDNum = SPrintF("%7d", $JDNum);
   $JD = SPrintF("% 8.1f", $JD);

   $BCADY = ($y > 0)? "AD $y":"BC ".abs($y);
   $BCAD  = ($y > 0)? 'AD':'BC';


// Construct output table line string.
   $wTable .= "$BCAD $YYYY-$Mmm-$dd-$DoWStr   $JDNum     $JD\n";
}

   $JGCalStr = ($JGMode == 'G')? '  Gregorian  ':'    Julian   ';
   $JGString = Str_Replace('-', ' ', trim($JGCalStr));
   $bcady = Str_Replace('-', ' ', trim($BCADY));

$wTable =
"  JULIAN DAY NUMBER AND JULIAN DATE TABLE

  For the Month of $mName, Year $bcady
  On the $JGString Calendar

-------------------------------------------
  $JGCalStr     JD Number   Julian Date
  Calendar Date     JD at 12h    JD at 00h
------------------  ---------   -----------
$wTable------------------  ---------   -----------
  $JGCalStr     JD at 12h    JD at 00h
  Calendar Date     JD Number   Julian Date
===========================================
";

// Adjust number of rows to use for current TEXTAREA.
   $TextRows = 15 + $mDays;

$TextArea =
"
<textarea rows='$TextRows' cols='44' ReadOnly
 title=' Double-Click to Select ALL Text '
 OnDblClick='this.select();' OnMouseUp='return true;'
 style='font-size:130%; font-weight:bold; padding:6px;
 border:2px solid blue; border-radius:8px;'>
$wTable</textarea>

";

   return $TextArea;
}
























   function JD_Num_Year_Table ($BCADYearNum, $JGCalMode='G')
{
   $y = IntVal(trim($BCADYearNum));
   $JGMode = (StrToUpper(substr(trim($JGCalMode),0,1)) <> 'J')? 'G':'J';

   if ($y < -4713 and $JGMode == 'J')
      {return "Invalid Year ($y) for Julian calendar.<br>Must be in the range from -4713 to 9999";}

   if ($y < -4714 and $JGMode == 'G')
      {return "Invalid Year ($y) for Gregorian calendar.<br>Must be in the range from -4714 to 9999";}

   $JDTable = '';
   $flag = FALSE;

   if ($JGMode == 'G' and $y == -4714) {$flag = TRUE;}

// Generate table of TEXTAREAs, one for each month of the year.
   for($m=1;   $m <= 12;   $m++)
      {
       if ($flag == TRUE and $m < 11)
          {
           $JDTable .= '';
          }
       else
          {
           $JDTable .= JD_Num_Month_Table ($y, $m, $JGMode)."<br><br>\n";
          }
      }

   return $JDTable;

}


?>



